class Tag < ApplicationRecord
  has_many :post_tags, dependent: :destroy
  has_many :posts, through: :post_tags
  has_many :tag_aliases, dependent: :destroy

  has_many :nico_tag_relations, foreign_key: :nico_tag_id, dependent: :destroy
  has_many :linked_tags, through: :nico_tag_relations, source: :tag

  has_many :reversed_nico_tag_relations, class_name: 'NicoTagRelation',
                                         foreign_key: :tag_id,
                                         dependent: :destroy
  has_many :linked_nico_tags, through: :reversed_nico_tag_relations, source: :nico_tag

  enum :category, { deerjikist:  'deerjikist',
                    meme:        'meme',
                    character:   'character',
                    general:     'general',
                    material:    'material',
                    nico:        'nico',
                    meta:        'meta' }

  validates :name, presence: true, length: { maximum: 255 }
  validates :category, presence: true, inclusion: { in: Tag.categories.keys }

  validate :nico_tag_name_must_start_with_nico

  scope :nico_tags, -> { where(category: :nico) }

  def self.tagme
    @tagme ||= Tag.find_or_initialize_by(name: 'タグ希望') do |tag|
      tag.category = 'meta'
    end
  end

  def self.bot
    @bot ||= Tag.find_or_initialize_by(name: 'bot操作') do |tag|
      tag.category = 'meta'
    end
  end

  private

  def nico_tag_name_must_start_with_nico
    if ((category == 'nico' && name&.[](0, 5) != 'nico:') || 
        (category != 'nico' && name&.[](0, 5) == 'nico:'))
      errors.add :name, 'ニコニコ・タグの命名規則に反してゐます．'
    end
  end
end
