import axios from 'axios'
import toCamel from 'camelcase-keys'
import { useEffect, useState } from 'react'
import { Helmet } from 'react-helmet-async'
import { Link } from 'react-router-dom'

import SectionTitle from '@/components/common/SectionTitle'
import TextArea from '@/components/common/TextArea'
import MainArea from '@/components/layout/MainArea'
import { API_BASE_URL, SITE_TITLE } from '@/config'

import type { NicoTag } from '@/types'


export default () => {
  const [nicoTags, setNicoTags] = useState<NicoTag[]> ([])
  const [editing, setEditing] = useState<{ [key: number]: boolean }> ({ })
  const [rawTags, setRawTags] = useState<{ [key: number]: string }> ({ })

  useEffect (() => {
    void (async () => {
      const res = await axios.get (`${ API_BASE_URL }/tags/nico`)
      const data = toCamel (res.data, { deep: true })

      setNicoTags (data.tags)

      const newEditing = Object.fromEntries (data.tags.map (t => [t.id, false]))
      setEditing (editing => ({ ...editing, ...newEditing }))

      const newRawTags = Object.fromEntries (
	  data.tags.map (t => [t.id, t.linkedTags.map (lt => lt.name).join (' ')]))
      setRawTags (rawTags => ({ ...rawTags, ...newRawTags }))
    }) ()
  }, [])

  return (
      <MainArea>
	<Helmet>
	  <title>ニコニコ連携 | {SITE_TITLE}</title>
	</Helmet>

	<div className="max-w-xl">
	  <SectionTitle>ニコニコ連携</SectionTitle>
	</div>

	<div className="mt-4">
	  <table className="table-auto w-full border-collapse">
	    <thead>
	      <tr>
		<th className="p-2 text-left">ニコニコタグ</th>
		<th className="p-2 text-left">連携タグ</th>
		<th></th>
	      </tr>
	    </thead>
	    <tbody>
	      {nicoTags.map (tag => (
		  <tr key={tag.id}>
		    <td className="p-2">
		      {tag.name}
		    </td>
		    <td className="p-2">
		      {editing[tag.id]
		       ? (
			  <TextArea value={rawTags[tag.id]} onChange={ev => {
			    setRawTags (rawTags => ({ ...rawTags, [tag.id]: ev.target.value }))
			  }} />)
		       : rawTags[tag.id]}
		    </td>
		    <td>
		      <a href="#" onClick={ev => {
			setEditing (editing => ({ ...editing, [tag.id]: !(editing[tag.id]) }))
		      }}>
			{editing[tag.id] ? <span className="text-red-400">更新</span> : '編輯'}
		      </a>
		    </td>
		  </tr>))}
	    </tbody>
	  </table>
	</div>
      </MainArea>)
}
