import axios from 'axios'
import toCamel from 'camelcase-keys'
import { useEffect, useState } from 'react'
import { Helmet } from 'react-helmet-async'
import { Link, useLocation, useNavigate, useParams } from 'react-router-dom'

import WikiBody from '@/components/WikiBody'
import PageTitle from '@/components/common/PageTitle'
import MainArea from '@/components/layout/MainArea'
import { API_BASE_URL, SITE_TITLE } from '@/config'
import { WikiIdBus } from '@/lib/eventBus/WikiIdBus'

import type { WikiPage } from '@/types'


export default () => {
  const { title } = useParams ()

  const location = useLocation ()
  const navigate = useNavigate ()

  const [wikiPage, setWikiPage] = useState<WikiPage | null | undefined> (undefined)

  const query = new URLSearchParams (location.search)
  const version = query.get ('version')

  useEffect (() => {
    if (/^\d+$/.test (title))
      {
        void (axios.get (`${ API_BASE_URL }/wiki/${ title }`)
              .then (res => navigate (`/wiki/${ res.data.title }`, { replace: true })))
        return
      }

    void (axios.get (`${ API_BASE_URL }/wiki/title/${ encodeURIComponent (title) }`, version && { params: { version } })
          .then (res => {
            setWikiPage (toCamel (res.data, { deep: true }))
            WikiIdBus.set (res.data.id)
          })
          .catch (() => setWikiPage (null)))

    return () => WikiIdBus.set (null)
  }, [title, location.search])

  return (
      <MainArea>
        <Helmet>
          <title>{`${ title } Wiki | ${ SITE_TITLE }`}</title>
        </Helmet>
        {(wikiPage && version) && (
            <div className="text-sm flex gap-3 items-center justify-center border border-gray-700 rounded px-2 py-1 mb-4">
              {wikiPage.pred ? (
                  <Link to={`/wiki/${ title }?version=${ wikiPage.pred }`}>
                    &lt; 古
                  </Link>) : <>(最古)</>}

              <span>{wikiPage.updatedAt}</span>

              {wikiPage.succ ? (
                  <Link to={`/wiki/${ title }?version=${ wikiPage.succ }`}>
                    新 &gt;
                  </Link>) : <>(最新)</>}
            </div>)}
        <PageTitle>{title}</PageTitle>
        <div className="prose mx-auto p-4">
          {wikiPage === undefined
           ? 'Loading...'
           : <WikiBody body={wikiPage?.body || `このページは存在しません。[新規作成してください](/wiki/new?title=${ title })。`} />}
        </div>
      </MainArea>)
}
